#'Save CSTools objects of class 's2dv_cube' containig experiment or oberved data in NCDF format
#'
#'@author Perez-Zanon Nuria, \email{nuria.perez@bsc.es}
#'
#'@description This function allows to save a object of class 's2dv_cube' in a .nc file.
#'
#'@param data an object of class \code{s2dv_cube}
#'@param destination a character string containing the file name in which to save the data with .nc extension.
#'@details It saves individual or a list of numerical and logical arrays. If an element is NULL, it won't be saved. Other classes will be stored in attributes.
#'@seealso \code{\link[easyNCDF]{ArrayToNc}}, \code{\link{SaveNC}} and \code{\link{CST_Load}}
#'

#'@import easyNCDF 
#'@import lubridate
#'@importFrom s2dverification InsertDim Subset
#'@examples
#'library(CSTools)
#'data <- lonlat_data$exp
#'destination <- "/home/Earth/nperez/CST_data/"
destination <- "/home/Earth/nperez/CST_data"

$DATASET$/$STORE_FREQ$_mean/$VAR_NAME$/$VAR_NAME$_$STAR_DATE$.nc
#'@export
SaveNC <- function(data, destination) {
  if (!is.character(destination) & length(destination) > 1) {
    stop("Parameter 'destination' must be a character string of one element ",
         "indicating the name of the file (including the folder if needed) ",
         "where the data will be saved.")
  }
  if (!inherits(data, 's2dv_cube')) {
    stop("Parameter 'data' must be of the class 's2dv_cube', ",
         "as output by CSTools::CST_Load.")
  }
dimname <- names(dim(data$data))
  if (is.null(dimnane)) {
    stop("Element 'data' in parameter 'data' must have named dimensions.")
  }
sdate_pos <- which(dimname == "sdate")

  if (length(sdate_pos) == 0) {
    stop("Element 'data' in parameter 'data' hasn't 'sdate' dimension.")
  } else if (length(sdate_pos) > 1) {
    stop("Element 'data' in parameter 'data' has more than one 'sdate'",
         " dimension.")
  }
n_sdates <- dim(data$data)[sdate_pos] # number of files to create

dataset_pos <- which(dimname == "dataset")
dims <- dim(data$data)
  if (length(dataset_pos) == 0) {
    warning("Element 'data' in parameter 'data' hasn't 'dataset' dimension.",
            "All data is stored in the same 'dataset' folder.")
    data$data <- InsertDim(var = data$data, posdim = 1, lendim = 1)
    dim(data$data)[1] <- "dataset"
    dimname <- c("dataset", dimname)
    dataset_pos = 1
  } else if (length(dataset_pos) > 1) {
    stop("Element 'data' in parameter 'data' has more than one 'dataset'",
         " dimension.")
  }
n_datasets <- dim(data$data)[dataset_pos] # number of folder by dataset

# dataset names:
datasets <- names(data$Datasets)
  if (n_datasets > length(datasets)) {
    warning("Dimension 'dataset' in element 'data' from parameter 'data' ",
            "is greater than those listed in element 'Datasets' and the ",
            "first element is reused.")
    datasets <- c(datasets, rep(datasets[1], n_datsets - length(datasets)))
  } else if (n_datasets < length(datasets)) {
    warning("Dimension 'dataset' in element 'data' from parameter 'data', ",
            "is smaller than those listed in element 'Datasets' and only the",
            " first element will be used.")
    datasets <- datasets[1 : n_datasets]
  }
# var names:
var_name <- data$Variable$varName
  if (length(var_name) != 1) {
    stop("One variable name must be included in element 'Variable$varName' ",
         "of parameter 'data'.") 
  }
  if (!is.character(var_name)) {
    stop("Element 'Variable$varName' of parameter 'data' ",
         "must be a character string.")
  }

# 1) create paths DONE
# 2) create folders DONE
# 3) select data DONE
# 4) create files
known_dim_names <- c("lat", "latitude", "lon", "longitude", "time", "ftime",
                     "sdate", "dataset", "nlevel", "levels")
dims_var <- NULL
list_pos <- 1
  if (any(dimname == 'latitude') | any(dimname == 'lat')) {
    dim_lat <- ncdim_def(name = 'lat', units = 'degrees',  vals = data$lat,
                         longname = 'latitude')
    dims_var[[list_pos]] <- dim_lat
    list_pos <- list_pos + 1
  }
  if (any(dimname == 'longitude') | any(dimname == 'lon')) {
    dim_lon <- ncdim_def(name = 'lon', units = 'degrees',  vals = data$lon,
                         longname = 'longitude')
    dims_var[[list_pos]] <- dim_lon
    list_pos <- list_pos + 1

  }
  if (any(dimname == 'time') | any(dimname == 'ftime')) {
    dim_time <- ncdim_def(name = 'time', units = 'days since 1970-01-01',
                          vals = as.numeric(as.Date(data$Dates$start))[1 : 3],
                          longname = 'time')
    dims_var[[list_pos]] <- dim_time
    list_pos <- list_pos + 1
  }
  if (any(dimname == 'level') | any(dimname =='level')) {
    stop("Ask for saving 3Dim fields to the mantainer.")
  }
  if (any(!(dimname %in% known_dim_names))) {
    dims_member <- dimname[!(dimname %in% known_dim_names)]
    if (length(dims_member) > 1) {
      stop("Ask for saving realizations fields to the mantainer.")
    } else {
      dim_memb <- ncdim_def(name = 'member', units = 'adimensinal',
                          vals = 1 : dim(data$data)[which(dimname == 'member')],
                          longname = 'member')
    dims_var[[list_pos]] <- dim_memb
    list_pos <- list_pos + 1
    }
  }

  for (i in 1 : n_datasets) {
    path <- file.path(destination, datasets[i], var_name)
    dir.create(path, recursive = TRUE)
    startdate <- gsub("-", "", data$Datasets[[i]]$InitializationDates[[1]])
    file_name <- paste0(var_name, "_", startdate, ".nc")
    full_filename <- file.path(path, file_name)

    data_dataset <- Subset(data$data, along = which(dimname == 'dataset'), indices = i)

      for (j in 1 : n_sdates) {
        n_data <- s2dverification::Subset(data_dataset, along = which(dimname == 'sdate'),
                         indices = j, drop = TRUE)
                           
        datanc  <- ncvar_def(name = var_name, units = attributes(data$Variable)$units,
                             dim = dims_var, missval = -99999)
        file_nc <- nc_create(full_filename, datanc)
        ncvar_put(file_nc, datanc, n_data)
        nc_close(file_nc)
      }
  }
}








length(data)
names(data)
dim(data$data)
dimnames <- dim(data$data)
data$Variable
data$Variable$varName
var_name <- data$Variable$varName
length(data$Datasets)
names(data$Datasets)
data$Dates
data$when
data$source_files



